@@ -73,9 +73,10 @@ class CLIParser:
         parser.add_argument(
             "--tag",
             "-t",
+            "--prefix",
             type=str,
             required=False,
-            help="the tag to use for tagging and naming entities",
+            help="the tag/prefix for naming or selecting entities",
             metavar="<tag>",
         )
         parser.add_argument(
@@ -241,12 +242,19 @@ class CLIParser:
                 "-i",
                 action="store_true",
                 required=False,
-                help="list, and interactively select, items to act on",
+                help="list, and interactively select, the items to act on",
             )
 
         if any(
             module in sys.argv[0]
-            for module in ["abort", "cancel", "delete", "shutdown", "terminate"]
+            for module in [
+                "abort",
+                "cancel",
+                "delete",
+                "shutdown",
+                "terminate",
+                "resize",
+            ]
         ):
             parser.add_argument(
                 "--yes",
@@ -256,6 +264,15 @@ class CLIParser:
                 help="perform destructive actions without requiring user confirmation",
             )
 
+        if any(module in sys.argv[0] for module in ["delete", "download"]):
+            parser.add_argument(
+                "--all",
+                "-a",
+                action="store_true",
+                required=False,
+                help="list all objects, at all levels in the prefix hierarchy",
+            )
+
         if "download" in sys.argv[0]:
             parser.add_argument(
                 "--yes",
@@ -269,24 +286,53 @@ class CLIParser:
                 "-d",
                 type=str,
                 required=False,
-                help="the directory to use for downloaded objects",
+                help=(
+                    "the directory to use for downloaded objects (defaults to the"
+                    " current directory)"
+                ),
                 metavar="<directory>",
             )
+            parser.add_argument(
+                "--flatten",
+                "-f",
+                action="store_true",
+                required=False,
+                help=(
+                    "flatten download paths (objects with the same name will be"
+                    " overwritten)"
+                ),
+            )
 
         if "list" in sys.argv[0]:
             parser.add_argument(
                 "--object-paths",
+                "--objects",
                 "-o",
                 action="store_true",
                 required=False,
                 help="list YellowDog Object Store object paths",
             )
             parser.add_argument(
-                "--tree",
-                "-T",
+                "--all",
+                "-a",
                 action="store_true",
                 required=False,
-                help="when used with '--object-paths', list the full object tree",
+                help=(
+                    "when used with '--objects', list all objects, not just the top"
+                    " level structure"
+                ),
+            )
+            parser.add_argument(
+                "--details",
+                "-d",
+                action="store_true",
+                required=False,
+                help=(
+                    "select objects from the list of matches and show their details"
+                    " (for Objects, Worker Pools, Compute Requirement Templates,"
+                    " Compute Source Templates, Keyrings, Machine Image Families,"
+                    " Namespace Storage Configurations)"
+                ),
             )
             parser.add_argument(
                 "--work-requirements",
@@ -304,7 +350,7 @@ class CLIParser:
             )
             parser.add_argument(
                 "--tasks",
-                "-a",
+                "-T",
                 action="store_true",
                 required=False,
                 help="list Tasks in selected Work Requirements",
@@ -328,24 +374,51 @@ class CLIParser:
                 "-l",
                 action="store_true",
                 required=False,
-                help="list live objects only",
+                help="list only active Compute Requirements/Worker Pools",
+            )
+            parser.add_argument(
+                "--compute-templates",
+                "-C",
+                action="store_true",
+                required=False,
+                help="list Compute Requirement Templates",
+            )
+            parser.add_argument(
+                "--source-templates",
+                "-S",
+                action="store_true",
+                required=False,
+                help="list Compute Source Templates",
+            )
+            parser.add_argument(
+                "--keyrings",
+                "-K",
+                action="store_true",
+                required=False,
+                help="list Keyrings",
+            )
+            parser.add_argument(
+                "--image-families",
+                "-I",
+                action="store_true",
+                required=False,
+                help="list Machine Image Families",
+            )
+            parser.add_argument(
+                "--namespaces",
+                "-N",
+                action="store_true",
+                required=False,
+                help="list Namespace Storage Configurations",
             )
 
         if "upload" in sys.argv[0]:
-            parser.add_argument(
-                "--directory",
-                "-d",
-                type=str,
-                required=False,
-                help="the Object Store directory name to use when uploading objects",
-                metavar="<directory>",
-            )
             parser.add_argument(
                 "--content-path",
                 "-C",
                 type=str,
                 required=False,
-                help="the path where the items to be uploaded are found",
+                help="the directory where the items to be uploaded are found",
                 metavar="<directory>",
             )
             parser.add_argument(
@@ -360,7 +433,10 @@ class CLIParser:
                 "-f",
                 action="store_true",
                 required=False,
-                help="suppress mirroring of local directory paths when uploading files",
+                help=(
+                    "don't mirror local directory structure when uploading files (files"
+                    " may be overwritten)"
+                ),
             )
             parser.add_argument(
                 "--recursive",
@@ -369,6 +445,16 @@ class CLIParser:
                 required=False,
                 help="recursively upload files from directories",
             )
+            parser.add_argument(
+                "--batch",
+                "-b",
+                action="store_true",
+                required=False,
+                help=(
+                    "use batch upload; file_patterns must contain wildcards and "
+                    "be quoted to prevent shell expansion"
+                ),
+            )
 
         if any(
             module in sys.argv[0] for module in ["submit", "provision", "instantiate"]
@@ -408,7 +494,8 @@ class CLIParser:
             )
 
         if any(
-            module in sys.argv[0] for module in ["submit", "provision", "instantiate"]
+            module in sys.argv[0]
+            for module in ["submit", "provision", "instantiate", "create"]
         ):
             parser.add_argument(
                 "--jsonnet-dry-run",
@@ -418,6 +505,114 @@ class CLIParser:
                 help="dry-run Jsonnet processing into JSON",
             )
 
+        if "resize" in sys.argv[0]:
+            parser.add_argument(
+                "worker_pool",
+                metavar="<worker-pool-or-compute-requirement-name-or-ID>",
+                type=str,
+                help=(
+                    "the name or YellowDog ID of the Worker Pool or Compute Requirement"
+                    " to resize"
+                ),
+            )
+            parser.add_argument(
+                "worker_pool_size",
+                metavar="<new-node/instance-count>",
+                type=int,
+                help="the desired number of (total) nodes in the Worker Pool",
+            )
+            parser.add_argument(
+                "--compute-requirement",
+                "-C",
+                action="store_true",
+                required=False,
+                help="resize a Compute Requirement instead of a Worker Pool",
+            )
+
+        if "shutdown" in sys.argv[0]:
+            parser.add_argument(
+                "worker_pool",
+                nargs="?",
+                default="",
+                metavar="<worker-pool-name-or-ID>",
+                type=str,
+                help="the name or YellowDog ID of the Worker Pool",
+            )
+
+        if "terminate" in sys.argv[0]:
+            parser.add_argument(
+                "compute_requirement",
+                nargs="?",
+                default="",
+                metavar="<compute-requirement-name-or-ID>",
+                type=str,
+                help="the name or YellowDog ID of the Compute Requirement",
+            )
+
+        if "cancel" in sys.argv[0]:
+            parser.add_argument(
+                "work_requirement",
+                nargs="?",
+                default="",
+                metavar="<work-requirement-name-or-ID>",
+                type=str,
+                help="the name or YellowDog ID of the Work Requirement",
+            )
+
+        if "delete" in sys.argv[0]:
+            parser.add_argument(
+                "object_paths_to_delete",
+                nargs="*",
+                default=[],
+                metavar="<object_path>",
+                type=str,
+                help="the object paths to delete; optional, overrides --tag/prefix",
+            )
+
+        if "download" in sys.argv[0]:
+            parser.add_argument(
+                "object_paths_to_download",
+                nargs="*",
+                default=[],
+                metavar="<object_path>",
+                type=str,
+                help="the object paths to download; optional, overrides --tag/prefix",
+            )
+        if any(module in sys.argv[0] for module in ["create", "remove"]):
+            parser.add_argument(
+                "resource_specifications",
+                nargs="+",
+                default=[],
+                metavar="<resource_specification>",
+                type=str,
+                help=(
+                    "the resource specifications to process (or resource IDs if used"
+                    " with 'yd-remove --ids')"
+                ),
+            )
+            parser.add_argument(
+                "--yes",
+                "-y",
+                action="store_true",
+                required=False,
+                help="allow updates without user confirmation",
+            )
+        if "create" in sys.argv[0]:
+            parser.add_argument(
+                "--show-keyring-passwords",
+                action="store_true",
+                required=False,
+                help="display YellowDog-generated password when creating a Keyring",
+            )
+
+        if "remove" in sys.argv[0]:
+            parser.add_argument(
+                "--ids",
+                action="store_true",
+                required=False,
+                help="remove resources using their YellowDog IDs (ydids)",
+            )
+
         self.args = parser.parse_args()
 
         if self.args.docs:
@@ -434,10 +629,8 @@ class CLIParser:
             print(
                 datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                 ":",
-                (
-                    "Note: the '--follow' ('-f') option is partially supported "
-                    "for Python versions 3.10 and above"
-                ),
+                "Note: the '--follow' ('-f') option is partially supported "
+                "for Python versions 3.10 and above",
             )
 
     @property
@@ -506,7 +699,12 @@ class CLIParser:
 
     @property
     def yes(self) -> Optional[bool]:
-        return self.args.yes
+        # This attribute is used in 'submit' without the '--yes' option
+        # being enabled for that module. Ensure that 'None' is returned.
+        try:
+            return self.args.yes
+        except AttributeError:
+            return None
 
     @property
     def object_paths(self) -> Optional[bool]:
@@ -537,8 +735,12 @@ class CLIParser:
         return self.args.live_only
 
     @property
-    def object_tree(self) -> Optional[bool]:
-        return self.args.tree
+    def all(self) -> Optional[bool]:
+        return self.args.all
+
+    @property
+    def details(self) -> Optional[bool]:
+        return self.args.details
 
     @property
     def debug(self) -> Optional[bool]:
@@ -564,6 +766,10 @@ class CLIParser:
     def recursive(self) -> Optional[bool]:
         return self.args.recursive
 
+    @property
+    def batch(self) -> Optional[bool]:
+        return self.args.batch
+
     @property
     def dry_run(self) -> Optional[bool]:
         return self.args.dry_run
@@ -612,6 +818,70 @@ class CLIParser:
     def pause_between_batches(self) -> Optional[bool]:
         return self.args.pause_between_batches
 
+    @property
+    def worker_pool_name(self) -> str:
+        return self.args.worker_pool
+
+    @property
+    def compute_requirement_name(self) -> str:
+        return self.args.compute_requirement
+
+    @property
+    def work_requirement_name(self) -> str:
+        return self.args.work_requirement
+
+    @property
+    def object_paths_to_delete(self) -> List[str]:
+        return self.args.object_paths_to_delete
+
+    @property
+    def object_paths_to_download(self) -> List[str]:
+        return self.args.object_paths_to_download
+
+    @property
+    def worker_pool_size(self) -> int:
+        return self.args.worker_pool_size
+
+    @property
+    def compute_req_resize(self) -> Optional[bool]:
+        return self.args.compute_requirement
+
+    @property
+    def flatten_download_paths(self) -> Optional[bool]:
+        return self.args.flatten
+
+    @property
+    def compute_templates(self) -> Optional[bool]:
+        return self.args.compute_templates
+
+    @property
+    def source_templates(self) -> Optional[bool]:
+        return self.args.source_templates
+
+    @property
+    def keyrings(self) -> Optional[bool]:
+        return self.args.keyrings
+
+    @property
+    def image_families(self) -> Optional[bool]:
+        return self.args.image_families
+
+    @property
+    def namespaces(self) -> Optional[bool]:
+        return self.args.namespaces
+
+    @property
+    def show_keyring_passwords(self) -> Optional[bool]:
+        return self.args.show_keyring_passwords
+
+    @property
+    def ids(self) -> Optional[bool]:
+        return self.args.ids
+
+    @property
+    def resource_specifications(self) -> List[str]:
+        return self.args.resource_specifications
+
 
 def lookup_module_description(module_name: str) -> Optional[str]:
     """
@@ -628,6 +898,8 @@ def lookup_module_description(module_name: str) -> Optional[str]:
         suffix = "aborting Tasks"
     elif "cancel" in module_name:
         suffix = "cancelling Work Requirements"
+    elif "create" in module_name:
+        suffix = "creating/updating assets"
     elif "download" in module_name:
         suffix = "downloading objects from the Object Store"
     elif "delete" in module_name:
@@ -642,6 +914,10 @@ def lookup_module_description(module_name: str) -> Optional[str]:
         suffix = "provisioning a Compute Requirement"
     elif "upload" in module_name:
         suffix = "uploading objects to the Object Store"
+    elif "remove" in module_name:
+        suffix = "removing assets"
+    elif "resize" in module_name:
+        suffix = "resizing Worker Pools"
 
     return None if suffix is None else prefix + suffix
 
