@@ -16,21 +16,23 @@ from yellowdog_client.model import (
 
 from yd_commands.config import (
     ARGS_PARSER,
+    WP_VARIABLES_PREFIX,
     ConfigWorkerPool,
     generate_id,
     link_entity,
     load_config_worker_pool,
 )
-from yd_commands.mustache import (
-    load_json_file_with_mustache_substitutions,
-    load_jsonnet_file_with_mustache_substitutions,
-)
 from yd_commands.printing import (
     print_compute_template_test_result,
     print_error,
     print_log,
     print_yd_object,
 )
+from yd_commands.provision_utils import get_user_data_property
+from yd_commands.variables import (
+    load_json_file_with_variable_substitutions,
+    load_jsonnet_file_with_variable_substitutions,
+)
 from yd_commands.wrapper import CLIENT, CONFIG_COMMON, main_wrapper
 
 
@@ -45,7 +47,7 @@ CONFIG_WP: ConfigWorkerPool = load_config_worker_pool()
 
 @main_wrapper
 def main():
-    # -C > -P > workerPoolData
+    # -C > -P > workerPoolData / computeRequirementData
     cr_json_file = (
         ARGS_PARSER.worker_pool_file
         if ARGS_PARSER.compute_requirement is None
@@ -54,13 +56,15 @@ def main():
     cr_json_file = (
         CONFIG_WP.worker_pool_data_file if cr_json_file is None else cr_json_file
     )
+    if cr_json_file is None:  # Finally, try 'computeRequirementData'
+        cr_json_file = CONFIG_WP.compute_requirement_data_file
 
-    # Use the Mustache prefix if this is a Worker Pool file
+    # Use the variable prefix if this is a Worker Pool file
     if cr_json_file is not None:
         if (
             ARGS_PARSER.worker_pool_file or CONFIG_WP.worker_pool_data_file
         ) and not ARGS_PARSER.compute_requirement:
-            prefix = "__"
+            prefix = WP_VARIABLES_PREFIX
         else:
             prefix = ""
         print_log(f"Loading Compute Requirement data from: '{cr_json_file}'")
@@ -68,7 +72,7 @@ def main():
         return
 
     if CONFIG_WP.template_id is None:
-        print_error("No 'templateId' supplied")
+        raise Exception("No 'templateId' supplied")
 
     if not ARGS_PARSER.report:
         print_log(
@@ -82,7 +86,7 @@ def main():
     )
 
     num_batches = len(batches)
-    if num_batches > 1:
+    if num_batches > 1 and not ARGS_PARSER.report:
         print_log(f"Batching into {num_batches} Compute Requirements")
 
     for batch_number in range(num_batches):
@@ -108,17 +112,21 @@ def main():
                 maintainInstanceCount=CONFIG_WP.maintainInstanceCount,
                 instanceTags=CONFIG_WP.instance_tags,
                 imagesId=CONFIG_WP.images_id,
-                userData=CONFIG_WP.user_data,
+                userData=get_user_data_property(CONFIG_WP),
             )
 
             if ARGS_PARSER.report:
                 print_log("Generating provisioning report only")
-                test_result: ComputeRequirementTemplateTestResult = (
-                    CLIENT.compute_client.test_compute_requirement_template(
-                        compute_requirement_template_usage
+                try:
+                    test_result: ComputeRequirementTemplateTestResult = (
+                        CLIENT.compute_client.test_compute_requirement_template(
+                            compute_requirement_template_usage
+                        )
                     )
-                )
-                print_compute_template_test_result(test_result)
+                    print_compute_template_test_result(test_result)
+                except requests.HTTPError as http_error:
+                    if "No sources" in http_error.response.text:
+                        print_log("No Compute Sources match the Template's constraints")
                 return
 
             if not ARGS_PARSER.dry_run:
@@ -127,6 +135,8 @@ def main():
                         compute_requirement_template_usage
                     )
                 )
+                if ARGS_PARSER.quiet:
+                    print(compute_requirement.name)
                 print_log(
                     f"Provisioned {link_entity(CONFIG_COMMON.url, compute_requirement)}"
                 )
@@ -147,8 +157,11 @@ def _allocate_nodes_to_batches(
     Helper function to distribute the number of requested instances
     as evenly as possible over Compute Requirements when batches are required.
     """
-    num_batches = ceil(initial_nodes / max_batch_size)
-    nodes_per_batch = floor(initial_nodes / num_batches)
+    try:
+        num_batches = ceil(initial_nodes / max_batch_size)
+        nodes_per_batch = floor(initial_nodes / num_batches)
+    except ZeroDivisionError:
+        return [CRBatch(target_instances=0)]
 
     # First pass population of batches with equal number of instances
     batches = [
@@ -193,18 +206,17 @@ def create_compute_requirement_from_json(cr_json_file: str, prefix: str = "") ->
     """
 
     if ARGS_PARSER.report:
-        print_error(
+        raise Exception(
             "Compute Template reports aren't available when using JSON "
             "Compute Requirement / Worker Pool specifications"
         )
-        return
 
     if cr_json_file.lower().endswith(".jsonnet"):
-        cr_data = load_jsonnet_file_with_mustache_substitutions(
+        cr_data = load_jsonnet_file_with_variable_substitutions(
             cr_json_file, prefix=prefix
         )
     else:
-        cr_data = load_json_file_with_mustache_substitutions(
+        cr_data = load_json_file_with_variable_substitutions(
             cr_json_file, prefix=prefix
         )
 
@@ -219,14 +231,16 @@ def create_compute_requirement_from_json(cr_json_file: str, prefix: str = "") ->
             # Generate a default name
             (
                 "requirementName",
-                CONFIG_WP.name
-                if CONFIG_WP.name is not None
-                else generate_id("cr" + "_" + CONFIG_COMMON.name_tag),
+                (
+                    CONFIG_WP.name
+                    if CONFIG_WP.name is not None
+                    else generate_id("cr" + "_" + CONFIG_COMMON.name_tag)
+                ),
             ),
             ("requirementNamespace", CONFIG_COMMON.namespace),
             ("requirementTag", CONFIG_COMMON.name_tag),
             ("templateId", CONFIG_WP.template_id),
-            ("userData", CONFIG_WP.user_data),
+            ("userData", get_user_data_property(CONFIG_WP)),
             ("imagesId", CONFIG_WP.images_id),
             ("instanceTags", CONFIG_WP.instance_tags),
             ("targetInstanceCount", CONFIG_WP.target_instance_count),
@@ -255,6 +269,8 @@ def create_compute_requirement_from_json(cr_json_file: str, prefix: str = "") ->
     name = cr_data["requirementName"]
     if response.status_code == 200:
         print_log(f"Provisioned Compute Requirement '{name}'")
+        if ARGS_PARSER.quiet:
+            print(name)
     else:
         print_error(f"Failed to provision Compute Requirement '{name}'")
         raise Exception(f"{response.text}")
