@@ -24,6 +24,7 @@ from yellowdog_client.model import (
     Task,
     TaskGroup,
     TaskInput,
+    TaskInputSource,
     TaskInputVerification,
     TaskOutput,
     TaskStatus,
@@ -253,8 +254,10 @@ def submit_work_requirement(
     )
     if not ARGS_PARSER.dry_run:
         work_requirement = CLIENT.work_client.add_work_requirement(work_requirement)
+        if ARGS_PARSER.quiet:
+            print(work_requirement.name)
         print_log(
-            f"Created "
+            "Created "
             f"{link_entity(CONFIG_COMMON.url, work_requirement)} "
             f"({work_requirement.name})"
         )
@@ -495,6 +498,12 @@ def add_tasks_to_task_group(
 
     # Iterate through batches
     for batch_number in range(num_task_batches):
+        if ARGS_PARSER.pause_between_batches and batch_number > 0:
+            print_log(
+                f"Pausing before submitting batch number {batch_number + 1}. "
+                "Press enter to continue:"
+            )
+            input()
         # Iterate through tasks in the batch
         for task_number in range(
             (TASK_BATCH_SIZE * batch_number),
@@ -580,12 +589,15 @@ def add_tasks_to_task_group(
                 optional_inputs_list,
             )
 
-            # Upload files in the 'inputs' list
+            # Upload files in the 'inputs' list, applying wildcard expansion.
             # (Duplicates won't be re-added)
+            expanded_files_list: List[str] = []
             for file in input_files_list:
-                UPLOADED_FILES.add_input_file(
+                expanded_files = UPLOADED_FILES.add_input_file(
                     filename=file, flatten_upload_paths=flatten_upload_paths
                 )
+                expanded_files_list += expanded_files
+            input_files_list = expanded_files_list
 
             # Upload files in the 'uploadFiles' list
             upload_files = check_list(
@@ -627,6 +639,8 @@ def add_tasks_to_task_group(
                 files=optional_inputs_list, verification=None, wr_name=ID
             )
 
+            inputs = deduplicate_inputs(inputs)
+
             # Set up the 'outputs' property
             outputs = [
                 TaskOutput.from_worker_directory(file_pattern=file, required=False)
@@ -717,9 +731,9 @@ def add_tasks_to_task_group(
         if not ARGS_PARSER.dry_run:
             if num_task_batches > 1:
                 print_log(
-                    f"Batch {str(batch_number + 1).zfill(len(str(num_task_batches)))} : "
-                    f"Added {len(tasks_list):,d} "
-                    f"Task(s) to Work Requirement Task Group '{task_group.name}'"
+                    f"Batch {str(batch_number + 1).zfill(len(str(num_task_batches)))} :"
+                    f" Added {len(tasks_list):,d} Task(s) to Work Requirement Task"
+                    f" Group '{task_group.name}'"
                 )
 
         # Empty the task list for the next batch
@@ -728,7 +742,8 @@ def add_tasks_to_task_group(
     if not ARGS_PARSER.dry_run:
         if num_tasks > 0:
             print_log(
-                f"Added a total of {num_tasks} Task(s) to Task Group '{task_group.name}'"
+                f"Added a total of {num_tasks} Task(s) to Task Group"
+                f" '{task_group.name}'"
             )
         else:
             print_log(f"No Tasks added to Task Group '{task_group.name}'")
@@ -790,7 +805,7 @@ def on_update(work_req: WorkRequirement):
         total += task_group.taskSummary.taskCount
     print_log(
         f"WORK REQUIREMENT is {work_req.status} with {completed}/{total} "
-        f"COMPLETED TASKS"
+        "COMPLETED TASKS"
     )
 
 
@@ -808,6 +823,30 @@ def cleanup_on_failure(work_requirement: WorkRequirement) -> None:
     UPLOADED_FILES.delete()
 
 
+def deduplicate_inputs(task_inputs: List[TaskInput]) -> List[TaskInput]:
+    """
+    Deduplicate a list of TaskInputs. This is useful when wildcards
+    are used. Note that TaskInputs that differ only in their verification
+    type will be caught by 'check_for_duplicates_in_file_lists()'.
+    """
+    deduplicated_task_inputs: List[TaskInput] = []
+    for task_input in task_inputs:
+        if task_input not in deduplicated_task_inputs:
+            deduplicated_task_inputs.append(task_input)
+        else:
+            namespace = (
+                CONFIG_COMMON.namespace
+                if task_input.source == TaskInputSource.TASK_NAMESPACE
+                else task_input.namespace
+            )
+            print_log(
+                f"Removing '{task_input.verification}' duplicate:"
+                f" '{namespace}::{task_input.objectNamePattern}'"
+            )
+
+    return deduplicated_task_inputs
+
+
 def check_for_duplicates_in_file_lists(*args: List[str]):
     """
     Tests for duplicates in file lists. If duplicates found, print an error
@@ -951,12 +990,12 @@ def create_task(
     ):
         flatten_input_paths = FlattenPath.FILE_NAME_ONLY
 
-    # Special processing for Bash tasks if the 'executable' property is set
-    # The Bash script is uploaded if this hasn't already been done, and
-    # added to the list of required files.
-    if task_type == "bash":
+    # Special processing for Bash, Python & PowerShell tasks if the 'executable'
+    # property is set. The script is uploaded if this hasn't already been done,
+    # and added to the list of required files.
+    if task_type in ["bash", "powershell", "python", "cmd", "bat"]:
         if executable is None:
-            print_log("Note: no 'executable' specified for 'bash' Task Type")
+            print_log(f"Note: no 'executable' specified for '{task_type}' Task Type")
             return _make_task(flatten_input_paths)
 
         UPLOADED_FILES.add_input_file(
@@ -975,16 +1014,19 @@ def create_task(
         # Avoid duplicate TaskInputs
         if task_input.objectNamePattern not in [x.objectNamePattern for x in inputs]:
             inputs.append(task_input)
+        executable_pathname = unique_upload_pathname(
+            filename=executable,
+            id=ID,
+            inputs_folder_name=INPUTS_FOLDER_NAME,
+            flatten_upload_paths=flatten_upload_paths,
+        )
+        if task_type in ["powershell", "cmd", "bat"]:
+            executable_pathname = executable_pathname.replace("/", "\\")
         args = [
-            unique_upload_pathname(
-                filename=executable,
-                id=ID,
-                inputs_folder_name=INPUTS_FOLDER_NAME,
-                flatten_upload_paths=flatten_upload_paths,
-            )
-            if flatten_input_paths is None
-            else basename(executable)
+            executable_pathname if flatten_input_paths is None else basename(executable)
         ] + args
+        if task_type in ["cmd", "bat"]:
+            args.insert(0, "/c")
         return _make_task(flatten_input_paths)
 
     # Special processing for Docker tasks if the 'executable property is set.
@@ -1098,6 +1140,8 @@ def submit_json_raw(wr_file: str):
     if response.status_code == 200:
         wr_id = jsons.loads(response.text)["id"]
         print_log(f"Created Work Requirement '{wr_name}' [{wr_id}]")
+        if ARGS_PARSER.quiet:
+            print(wr_name)
     else:
         print_error(f"Failed to create Work Requirement '{wr_name}'")
         raise Exception(f"{response.text}")
@@ -1121,15 +1165,16 @@ def submit_json_raw(wr_file: str):
 
     # Warn about VERIFY_AT_START files & halt to allow upload or
     # Work Requirement cancellation
-    if len(verify_at_start_files) != 0:
+    if ARGS_PARSER.quiet is False and len(verify_at_start_files) != 0:
         print_log(
             "The following files may be required ('VERIFY_AT_START') "
             "before Tasks are submitted, or the Tasks will fail."
         )
         print_log(
             "You now have an opportunity to upload the required files "
-            "before Tasks are submitted:\n"
+            "before Tasks are submitted:"
         )
+        print()
         print_numbered_strings(sorted(list(verify_at_start_files)))
         if not confirmed("Proceed now (y), or Cancel Work Requirement (n)?"):
             print_log(f"Cancelling Work Requirement '{wr_name}'")
@@ -1151,7 +1196,9 @@ def submit_json_raw(wr_file: str):
                     f"taskGroups/{task_group_name}/tasks"
                 ),
                 headers={
-                    "Authorization": f"yd-key {CONFIG_COMMON.key}:{CONFIG_COMMON.secret}"
+                    "Authorization": (
+                        f"yd-key {CONFIG_COMMON.key}:{CONFIG_COMMON.secret}"
+                    )
                 },
                 json=task_batch,
             )
